/* -*-c-*-
 *
 * $Id: main.c,v 1.1 2002/01/25 19:34:45 mdw Exp $
 *
 * Main program
 *
 * (c) 2001 Mark Wooding
 */

/*----- Licensing notice --------------------------------------------------* 
 *
 * This file is part of Jog: Programming for a jogging machine.
 *
 * Jog is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Jog is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jog; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*----- Revision history --------------------------------------------------* 
 *
 * $Log: main.c,v $
 * Revision 1.1  2002/01/25 19:34:45  mdw
 * Initial revision
 *
 */

/*----- Header files ------------------------------------------------------*/

#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include <sys/time.h>
#include <unistd.h>

#include <mLib/alloc.h>
#include <mLib/mdwopt.h>
#include <mLib/quis.h>
#include <mLib/report.h>
#include <mLib/trace.h>

#include "err.h"
#include "rxglue.h"
#include "txport.h"
#include "tx-serial-unix.h"

/*----- Shutdown stuff ----------------------------------------------------*/

static int sigtab[] = { SIGINT, SIGQUIT, SIGTERM, SIGHUP, -1 };

static void tidy(void)
{
  txsu_shutdown();
}

static void sigtidy(int sig)
{
  tidy();
  signal(sig, SIG_DFL);
  raise(sig);
}

/*----- Help functions ----------------------------------------------------*/

static void usage(FILE *fp)
{
  pquis(fp, "Usage: $ [-t TRANSPORT] [-f FILE] [-c CONFIG] SCRIPT [ARG]\n");
}

static void version(FILE *fp)
{
  pquis(fp, "$, version " VERSION "\n");
}

static void help(FILE *fp)
{
  version(fp);
  fputc('\n', fp);
  usage(fp);
  fputs("\n\
Options provided:\n\
\n\
-h, --help		Print this help message.\n\
-v, --version		Show the version number.\n\
-u, --usage		Show terse usage summary.\n\
\n\
-t, --transport=NAME	Use transport type NAME.\n\
-f, --tx-file=FILE	Communicate using the named FILE.\n\
-c, --tx-config=CONFIG	Use CONFIG as transport configuration.\n\
",
	fp);
}

/*----- Main code ---------------------------------------------------------*/

int main(int argc, char *argv[])
{
  unsigned f = 0;
  int rc = 0;
  int i;

#define f_bogus 1u

  ego(argv[0]);
  atexit(tidy);
  for (i = 0; sigtab[i] >= 0; i++)
    signal(sigtab[i], sigtidy);

  err_init();
  rx_init();
  trace_on(stderr, 0u);
  if ((txname = getenv("JOGTX")) != 0)
    ;
  else
    txname = txlist->name;

  for (;;) {
    static const struct option opt[] = {

      /* --- Standard help options --- */

      { "help",		0,		0,	'h' },
      { "version",	0,		0,	'v' },
      { "usage",	0,		0,	'u' },

      /* --- Transport configuration stuff --- */

      { "transport",	OPTF_ARGREQ,	0,	't' },
      { "transport-config",
			OPTF_ARGREQ,	0,	'c' },
      { "tx-config",	OPTF_ARGREQ,	0,	'c' },
      { "txconfig",	OPTF_ARGREQ,	0,	'c' },
      { "config",	OPTF_ARGREQ,	0,	'c' },
      { "transport-file",
			OPTF_ARGREQ,	0,	'f' },
      { "tx-file",	OPTF_ARGREQ,	0,	'f' },
      { "txfile",	OPTF_ARGREQ,	0,	'f' },
      { "file",		OPTF_ARGREQ,	0,	'f' },

      /* --- End marker --- */

      { 0,		0,		0,	0 }
    };

    i = mdwopt(argc, argv, "hvut:c:f:", opt, 0, 0, 0);
    if (i < 0)
      break;

    switch (i) {

      /* --- Standard help options --- */

      case 'h':
	help(stdout);
	exit(0);
      case 'v':
	version(stdout);
	exit(0);
      case 'u':
	usage(stdout);
	exit(0);

      /* --- Transport configuration stuff --- */

      case 't':
	txname = optarg;
	break;
      case 'c':
	txconf = optarg;
	break;
      case 'f':
	txfile = optarg;
	break;

      /* --- Errors --- */

      default:
	f |= f_bogus;
	break;
    }
  }

  if ((f & f_bogus) || (optind != argc - 1 && optind != argc - 2)) {
    usage(stderr);
    exit(EXIT_FAILURE);
  }

  rc = rx_runfile(argv[optind],
		  argc - optind - 1, (const char *const *)argv + optind + 1);
  return (rc ? EXIT_FAILURE : 0);
}

/*----- That's all, folks -------------------------------------------------*/
