/* -*-c-*-
 *
 * $Id: serial.h,v 1.2 2002/01/30 09:23:58 mdw Exp $
 *
 * Common serial functionality
 *
 * (c) 2001 Mark Wooding
 */

/*----- Licensing notice --------------------------------------------------* 
 *
 * This file is part of Jog: Programming for a jogging machine.
 *
 * Jog is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Jog is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jog; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*----- Revision history --------------------------------------------------* 
 *
 * $Log: serial.h,v $
 * Revision 1.2  2002/01/30 09:23:58  mdw
 * Follow new transport configuration interface.  Add parameters for flow
 * control.  Break the format into separate parameters for independent
 * configuration.
 *
 * Revision 1.1  2002/01/25 19:34:45  mdw
 * Initial revision
 *
 */

#ifndef SERIAL_H
#define SERIAL_H

#ifdef __cplusplus
  extern "C" {
#endif

/*----- Header files ------------------------------------------------------*/

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

/*----- Data structures ---------------------------------------------------*/

typedef struct serial_config {
  unsigned long baud;			/* Baud rate */
  unsigned char wordlen;		/* Word length */
  unsigned char parity;			/* Parity type (magic constant) */
  unsigned char stopbits;		/* Number of stop bits */
  unsigned char flow;			/* Flow control */
} serial_config;

enum { PARITY_NONE, PARITY_ODD, PARITY_EVEN };
enum { FLOW_NONE, FLOW_XONXOFF, FLOW_RTSCTS };

#define SERIAL_INIT { 9600, 8, PARITY_NONE, 1, FLOW_NONE }

/*----- Functions provided ------------------------------------------------*/

/* --- @serial_parse@ --- *
 *
 * Arguments:	@serial_config *sc@ = pointer to serial config structure
 *		@const char *k, *v@ = keyword and value pair
 *
 * Returns:	Zero if OK, or @-1@ on error.
 *
 * Use:		Parses a serial config string into something more
 *		reasonable.  The config structure should have been
 *		initialized to something sensible (e.g., @SERIAL_INIT@)
 *		already.
 */

extern int serial_parse(serial_config */*sc*/,
			const char */*k*/, const char */*v*/);

/*----- That's all, folks -------------------------------------------------*/

#ifdef __cplusplus
  }
#endif

#endif
